#ifndef ComplexNumber_H
#define ComplexNumber_H
/*########################################################################
	File:			ComplexNumber.h
	Creation date:	25th May 2002

	Copyright 2002 Mark Williams, all rights reserved.
	Contact: mark@extrabit.com
	http://www.extrabit.com

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

########################################################################*/

/*########################################################################
	Headers
########################################################################*/

/*########################################################################
	Declarations
########################################################################*/

/*########################################################################
	Class Definition
########################################################################*/

/**	Stores a complex number z = x + iy
 *	where 'x' is the real component
 *	and 'y' is the imaginary component.
 *	Provides various mathematical operations for complex numbers.
 */
class ComplexNumber
{
public:
	ComplexNumber (void) { m_x = m_y = 0.0 ; }
	ComplexNumber (double x, double y) { m_x = x ; m_y = y ; }

	const double& x (void) const { return m_x ; }
	const double& y (void) const { return m_y ; }
	double& x (void) { return m_x ; }
	double& y (void) { return m_y ; }

	double inq_magnitude2 (void) const ;
	ComplexNumber inq_square (void) const ;
	ComplexNumber inq_cube (void) const ;
	ComplexNumber inq_inverse (void) const ;

	const ComplexNumber& operator+= (const ComplexNumber&) ;
	const ComplexNumber& operator-= (const ComplexNumber&) ;
	ComplexNumber operator+ (const ComplexNumber&) const ;
	ComplexNumber operator- (const ComplexNumber&) const ;
	ComplexNumber operator* (const ComplexNumber&) const ;

private:
	/**	Real component. */
	double m_x ;

	/**	Imaginary component. */
	double m_y ;
} ;


/**	@return Returns the square of the magnitude of this ComplexNumber.
 */
inline double ComplexNumber::inq_magnitude2 (void) const
{
	return m_x*m_x + m_y*m_y ;
}


/**	@return Returns the square of this ComplexNumber.
 */
inline ComplexNumber ComplexNumber::inq_square (void) const
{
	return ComplexNumber (m_x*m_x - m_y*m_y, 2.0*m_x*m_y) ;
}


/**	@return Returns the cube of this ComplexNumber.
 */
inline ComplexNumber ComplexNumber::inq_cube (void) const
{
	double x2 = m_x*m_x ;
	double y2 = m_y*m_y ;
	return ComplexNumber (m_x*x2 - 3.0*m_x*y2, -m_y*y2 + 3.0*x2*m_y) ;
}


/**	@return Returns the recipricol of this ComplexNumber.
 */
inline ComplexNumber ComplexNumber::inq_inverse (void) const
{
	double a = m_x*m_x + m_y*m_y ;
	return ComplexNumber (m_x/a, -m_y/a) ;
}


/**	Adds the given ComplexNumber to this one.
 */
inline const ComplexNumber& ComplexNumber::operator+= (const ComplexNumber& z)
{
	m_x += z.m_x ;
	m_y += z.m_y ;
	return *this ;
}


/**	Subtracts the given ComplexNumber to this one.
 */
inline const ComplexNumber& ComplexNumber::operator-= (const ComplexNumber& z)
{
	m_x -= z.m_x ;
	m_y -= z.m_y ;
	return *this ;
}


/**	@return Returns the sum of this ComplexNumber and the given one.
 */
inline ComplexNumber ComplexNumber::operator+ (const ComplexNumber& z) const
{
	return ComplexNumber (m_x + z.m_x, m_y + z.m_y) ;
}


/**	@return Returns the difference between this ComplexNumber and the given one.
 */
inline ComplexNumber ComplexNumber::operator- (const ComplexNumber& z) const
{
	return ComplexNumber (m_x - z.m_x, m_y - z.m_y) ;
}


/**	@return Returns the product of this ComplexNumber and the given one.
 */
inline ComplexNumber ComplexNumber::operator* (const ComplexNumber& z) const
{
	return ComplexNumber (m_x*z.m_x - m_y*z.m_y, m_x*z.m_y + m_y*z.m_x) ;
}


/**	@return Returns the product of this ComplexNumber and the given scalar.
 */
inline ComplexNumber operator* (const ComplexNumber& z, double s)
{
	return ComplexNumber (s*z.x(), s*z.y()) ;
}


/**	@return Returns the product of this ComplexNumber and the given scalar.
 */
inline ComplexNumber operator* (double s, const ComplexNumber& z)
{
	return ComplexNumber (s*z.x(), s*z.y()) ;
}


/*########################################################################
	End
########################################################################*/
#endif
