/*########################################################################
	File:			FastImageGen.cpp
	Creation date:	6th June 2002

	Copyright 2002 Mark Williams, all rights reserved.
	Contact: mark@extrabit.com
	http://www.extrabit.com

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

########################################################################*/

/*########################################################################
	Headers
########################################################################*/

#include "StdAfx.h"

#include "Processor.h"
#include "Palette.h"
#include "FractalImage.h"
#include "FastImageGen.h"

/*########################################################################
	Local declarations
########################################################################*/

/*########################################################################
	Local definitions
########################################################################*/


/**	Constructor.
 *	@param image The image that this FastImageGen will generate.
 */
FastImageGen::FastImageGen (FractalImage* image)
{
	m_image = image ;
	m_done = false ;
	m_progress = 0 ;
	m_x = 0 ;
	m_y = 0 ;
	m_curr_pixel_itns = 0 ;
	m_curr_rgb_pixel = 0 ;
}


/**	Overriden from ImageGen base class.
 */
void FastImageGen::init (void)
{
	// No initialisation needed.
}


/**	Overriden from ImageGen base class.
 *	Initialises all the member variables for processing 'm_image'.
 *	This must be called before process() is called, and whenever
 *	the image changes.
 */
void FastImageGen::restart (void)
{
	// Calculate 'm_xoffs' and 'm_yoffs'.
	Vector2 org   (0.0, 0.0) ;
	Vector2 org_x (1.0, 0.0) ;
	Vector2 org_y (0.0, 1.0) ;
	const Matrix33& view_transform = m_image->inq_view_transform() ;
	view_transform.transform (&org) ;
	view_transform.transform (&org_x) ;
	view_transform.transform (&org_y) ;
	m_xoffs = org_x - org ;
	m_yoffs = org_y - org ;

	// Set 'm_rowstart' to the centre of the centre of the pixel in the first (bottom) row.
	m_rowstart = Vector2 (0.5, 0.5) ;
	view_transform.transform (&m_rowstart) ;

	m_curr_pixel = m_rowstart ;
	m_curr_rgb_pixel = m_image->inq_rgb_pixels() ;
	m_curr_pixel_itns = m_image->inq_pixel_itns() ;

	m_done = false ;
	m_progress = 0 ;
	m_x = 0 ;
	m_y = 0 ;
}


/**	Overriden from ImageGen base class.
 *	Processes the next pixel.
 *	This method should be called repeatedly until inq_done() returns true.
 *	@return Returns true if the display should be updated (which is when
 *	all pixels have been processed), otherwise false.
 */
bool FastImageGen::process (void)
{
	if (m_done) return false ;

	ComplexNumber initial_z = m_image->inq_initial_z() ;
	const rgbcolour_t* colours = m_image->inq_palette()->inq_colours() ;
	unsigned short max_itns = (unsigned short)(m_image->inq_palette()->inq_ncolours() - 1) ;
	ComplexNumber z = m_image->convert_point_to_z (m_curr_pixel) ;
	unsigned short itns = m_image->inq_processor()->iterate_to_end (initial_z, z, max_itns) ;
	*m_curr_pixel_itns++ = itns ;
	*m_curr_rgb_pixel++ = colours[itns] ;
	m_curr_pixel += m_xoffs ;
	if (++m_x == m_image->inq_width())
	{
		m_x = 0 ;
		m_rowstart += m_yoffs ;
		m_curr_pixel = m_rowstart ;
		if (++m_y == m_image->inq_height())
		{
			m_done = true ;
		}

		// Update progress counter - just based on current line.
		m_progress = (int)(100.0*(double)m_y/(double)m_image->inq_height()) ;
	}


	return m_done ;
}


/**	Overriden from ImageGen base class.
 *	@return Returns true when image has been generated.
 */
bool FastImageGen::inq_done (void) const
{
	return m_done ;
}


/**	Overriden from ImageGen base class.
 *	@return Returns the value shown as a progress indicator in the status bar.
 */
int FastImageGen::inq_progress (void) const
{
	return m_progress ;
}


/*########################################################################
	End
########################################################################*/
