/*########################################################################
	FracWnd.cpp

	Copyright 2002 Mark Williams, all rights reserved.
	Contact: mark@extrabit.com
	http://www.extrabit.com

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

########################################################################*/

#include "stdafx.h"
#include "FractalViewer.h"

#include "Palette.h"
#include "FractalImage.h"

#include "FracWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CFracWnd


/////////////////////////////////////////////////////////////////////////////
CFracWnd::CFracWnd()
{
	m_image = 0 ;
	m_rgb_bitmap = NULL ;
	m_in_drag = false ;
}


/////////////////////////////////////////////////////////////////////////////
CFracWnd::~CFracWnd()
{
    ::DeleteObject(m_rgb_bitmap);
}


/////////////////////////////////////////////////////////////////////////////
void CFracWnd::set_image (FractalImage* image)
{
	m_image = image ;
}


/**	Creates a cached image of the given size (in pixels), for drawing into
 *	the given CDC.
 */
void CFracWnd::construct (CDC* p_dc, int width, int height)
{
	static const int BPS = 8 ;		// Bits per sample.
	static const int SPP = 4 ;		// Samples per pixel.

	BITMAPINFO bmi;

	bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bmi.bmiHeader.biPlanes = 1;					// always 1 plane
	bmi.bmiHeader.biCompression = BI_RGB;           
	bmi.bmiHeader.biBitCount = BPS * SPP;		// bits bits per pixel 
	bmi.bmiHeader.biSizeImage = 0;				// size in bytes of image 
	bmi.bmiHeader.biXPelsPerMeter = 5000;
	bmi.bmiHeader.biYPelsPerMeter = 5000;
	bmi.bmiHeader.biClrUsed = 0;				// use all colour indices 
	bmi.bmiHeader.biClrImportant = 0;			// all colours important 

	// this is the bitmap we need to create
	// with origin at the bottom left
	bmi.bmiHeader.biWidth = (LONG)width;
	bmi.bmiHeader.biHeight = (LONG)height;

    void* rgb_pixels = 0;

	// allocate the bitmap with a pointer to the bitmap bits
	HBITMAP hdibbitmap = CreateDIBSection(
		p_dc->m_hDC,
		&bmi,
		DIB_RGB_COLORS,
		&rgb_pixels,
		NULL, 
		0);

    // here's the new bitmaps bits and handle
    m_rgb_bitmap = hdibbitmap;
	m_image->resize (width, height, (rgbcolour_t*)rgb_pixels) ;
	((CFractalViewerApp*)AfxGetApp())->image_resized() ;
}


/**	Draws the image into the given DC.
 *	@param image_rect_v The rectangle into which the image should be fitted,
 */
void CFracWnd::draw_cbitmap (CDC* p_dc,
							 const CRect& image_rect_v,
							 HBITMAP bitmap, DWORD rasterop)
{
	HDC mem_dc = ::CreateCompatibleDC (p_dc->m_hDC) ;
	ASSERT (mem_dc != 0) ;
	HBITMAP old_bitmap = (HBITMAP)::SelectObject (mem_dc, bitmap) ;
	ASSERT (old_bitmap != 0) ;

	int image_width  = m_image->inq_width() ;
	int image_height = m_image->inq_height() ;
	if (image_rect_v.Width()  == image_width &&
		image_rect_v.Height() == image_height)
	{
		int s_ix = 0 ;
		int s_iy = 0 ;
		::BitBlt (p_dc->m_hDC,
					image_rect_v.left,
					image_rect_v.top,
					image_width,
					image_height,
					mem_dc, s_ix, s_iy, rasterop) ;
	}
	else
	{
		double scale_x = (double)image_width  / image_rect_v.Width() ;
		double scale_y = (double)image_height / image_rect_v.Height() ;
		int s_ix = 0 ;
		int s_iy = 0 ;
		int s_iw = image_width ;
		int s_ih = image_height ;

		if (s_iw > 0 && s_ih > 0)
		{
			::SetStretchBltMode (p_dc->m_hDC, COLORONCOLOR) ;

			::StretchBlt (p_dc->m_hDC,
				image_rect_v.left,
				image_rect_v.top,
				image_rect_v.Width(),
				image_rect_v.Height(),
				mem_dc, 
				s_ix, 
				s_iy, 
				s_iw, 
				s_ih, 
				rasterop) ;
		}
	}

	::SelectObject (mem_dc, old_bitmap) ;
	::DeleteDC (mem_dc) ;
}


/**	Sets the view transform to show the given rectangle in window space
 *	(which has (0,0) at top left).
 */
void CFracWnd::zoom_in (const CRect& zoom_rect)
{
	// Need to convert the given rectangle in window coordinate which has y=0 at the top
	// to FractalImage coords, which has y=0 at the bottom.
	CRect client_rect ;
	GetClientRect (&client_rect) ;
	int view_width  = client_rect.Width() ;
	int view_height = client_rect.Height() ;

	Vector2 bl (zoom_rect.left,  view_height - zoom_rect.bottom) ;
	Vector2 br (zoom_rect.right, view_height - zoom_rect.bottom) ;
	Vector2 tl (zoom_rect.left,  view_height - zoom_rect.top) ;
	const Matrix33& transform = m_image->inq_view_transform() ;
	transform.transform (&bl) ;
	transform.transform (&br) ;
	transform.transform (&tl) ;

	Vector2 centre = (tl + br) / 2.0 ;
	double zoom_width = bl.distance_to (br) ;
	double zoom_height = bl.distance_to (tl) ;
	if (zoom_width*view_height < zoom_height*view_width)
	{
		// Zoom rectangle aspect ratio is narrower than view,
		// so use height to determine new view.
		zoom_width = zoom_height*view_width/(double)view_height ;
	}

	m_image->set_view (centre, zoom_width, m_image->inq_view_angle()) ;
	((CFractalViewerApp*)AfxGetApp())->restart() ;
	((CFractalViewerApp*)AfxGetApp())->update_status_bar() ;
}


BEGIN_MESSAGE_MAP(CFracWnd, CWnd)
	//{{AFX_MSG_MAP(CFracWnd)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CFracWnd message handlers

/////////////////////////////////////////////////////////////////////////////
void CFracWnd::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

	CRect rect ;
	GetClientRect (rect) ;

	int width  = rect.Width() ;
	int height = rect.Height() ;
	if (width == 0 || height == 0) return ;

	if (m_rgb_bitmap == NULL || width != m_image->inq_width() || height != m_image->inq_height())
	{
		// Need to sort out resizing in OnPaint, since a CDC is needed to
		// create an offscreen bitmap to draw into.
		if (m_rgb_bitmap) ::DeleteObject(m_rgb_bitmap);
		m_rgb_bitmap = 0 ;
		construct (&dc, width, height) ;
	}

	draw_cbitmap (&dc, rect, m_rgb_bitmap, SRCCOPY) ;

	if (m_in_drag)
	{
		CRect bb_rect (m_mouse_down, m_mouse_curr) ;
		bb_rect.NormalizeRect() ;

		COLORREF gray = RGB(128,128,128) ;
		CBrush black_brush (gray) ;
		dc.FrameRect (bb_rect, &black_brush) ;
	}
}


/////////////////////////////////////////////////////////////////////////////
void CFracWnd::OnLButtonDown(UINT nFlags, CPoint point) 
{
	SetCapture() ;

	m_mouse_down = point ;
	m_mouse_curr = point ;
	m_in_drag = true ;

	CWnd::OnLButtonDown(nFlags, point);
}


/////////////////////////////////////////////////////////////////////////////
void CFracWnd::OnLButtonUp(UINT nFlags, CPoint point) 
{
	ReleaseCapture() ;

	if (m_in_drag)
	{
		CRect rect (m_mouse_down, m_mouse_curr) ;
		rect.NormalizeRect() ;

		// Make sure rectangle has been dragged out to at least MIN_ZOOM_PIXELS.
		const int MIN_ZOOM_PIXELS = 4 ;
		if (rect.Width() >= MIN_ZOOM_PIXELS)
		{
			zoom_in (rect) ;
		}
		m_in_drag = false ;
	}

	CWnd::OnLButtonUp(nFlags, point);
}


/////////////////////////////////////////////////////////////////////////////
void CFracWnd::OnMouseMove(UINT nFlags, CPoint point) 
{
	if (m_in_drag)
	{
		CRect old_rect (m_mouse_down, m_mouse_curr) ;
		old_rect.NormalizeRect() ;

		m_mouse_curr = point ;
		CRect new_rect (m_mouse_down, m_mouse_curr) ;
		new_rect.NormalizeRect() ;

		CRect rect ;
		rect.UnionRect (old_rect, new_rect) ;
		rect.InflateRect (1, 1) ;
		InvalidateRect (rect) ;
	}

	CWnd::OnMouseMove(nFlags, point);
}


/////////////////////////////////////////////////////////////////////////////
void CFracWnd::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);
	
	if (cx > 0 && cy > 0)
	{
		//((CFractalViewerApp*)AfxGetApp())->update_view() ;
	}
}
