/*########################################################################
	File:			Matrix33.cpp
	Creation date:	25th May 2002

	Copyright 2002 Mark Williams, all rights reserved.
	Contact: mark@extrabit.com
	http://www.extrabit.com

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

########################################################################*/

/*########################################################################
	Headers
########################################################################*/

#include "StdAfx.h"

#include <math.h>

#include "Vector2.h"
#include "Matrix33.h"

/*########################################################################
	Local declarations
########################################################################*/

/*########################################################################
	Local definitions
########################################################################*/


/**	Default constructor.
 *	Initialises the matrix to all 0s.
 */
Matrix33::Matrix33 (void)
{
	for (int r = 0 ; r < 3 ; r++)
	{
		for (int c = 0 ; c < 3 ; c++)
		{
			m_elements[r][c] = 0.0 ;
		}
	}
}


/**	Constructor for operations that take no arguments.
 *	The first argument should be one of the following values:
 *		IDENTITY
 *		ZERO
 */
Matrix33::Matrix33 (Matrix33_init type)
{
	switch (type)
	{
	case IDENTITY: set_identity() ; break ;
	case ZERO: set_zero() ; break ;
	}
}


/**	Constructor for operations that are defined by a double.
 *	The first argument should be one of the following values:
 *		ROTATE
 *		SCALE
 */
Matrix33::Matrix33 (Matrix33_init_d type, double value)
{
	switch (type)
	{
	case ROTATE: set_rotation (value) ; break ;
	case SCALE: set_scale (value) ; break ;
	}
}


/**	Constructor for operations that are defined by a ZVector2.
 *	The first argument should be one of the following values:
 *		TRANSLATE
 *		SCALE_XY
 */
Matrix33::Matrix33 (Matrix33_init_v type, const Vector2& value)
{
	switch (type)
	{
	case TRANSLATE: set_translation (value) ; break ;
	case SCALE_XY: set_scale (value) ; break ;
	}
}


/**	Sets this matrix to the zero matrix.
 */
void Matrix33::set_zero (void)
{
	m_elements[0][0] = 0.0 ;
	m_elements[0][1] = 0.0 ;
	m_elements[0][2] = 0.0 ;

	m_elements[1][0] = 0.0 ;
	m_elements[1][1] = 0.0 ;
	m_elements[1][2] = 0.0 ;

	m_elements[2][0] = 0.0 ;
	m_elements[2][1] = 0.0 ;
	m_elements[2][2] = 0.0 ;
}


/**	Sets this matrix to the identity matrix.
 */
void Matrix33::set_identity (void)
{
	m_elements[0][0] = 1.0 ;
	m_elements[0][1] = 0.0 ;
	m_elements[0][2] = 0.0 ;

	m_elements[1][0] = 0.0 ;
	m_elements[1][1] = 1.0 ;
	m_elements[1][2] = 0.0 ;

	m_elements[2][0] = 0.0 ;
	m_elements[2][1] = 0.0 ;
	m_elements[2][2] = 1.0 ;
}


/**	Sets the elements of this matrix so that it represents a 2D translation
 *	of the given vector.
 */
void Matrix33::set_translation (const Vector2& v)
{
	m_elements[0][0] = 1.0 ;
	m_elements[0][1] = 0.0 ;
	m_elements[0][2] = v.x() ;

	m_elements[1][0] = 0.0 ;
	m_elements[1][1] = 1.0 ;
	m_elements[1][2] = v.y() ;

	m_elements[2][0] = 0.0 ;
	m_elements[2][1] = 0.0 ;
	m_elements[2][2] = 1.0 ;
}


/**	Sets the elements of this matrix so that it represents a 2D rotation
 *	about the origin by the given anti-clockwise angle in radians.
 */
void Matrix33::set_rotation (double a)
{
	double cosa = cos (a) ;
	double sina = sin (a) ;

	m_elements[0][0] = cosa ;
	m_elements[0][1] = -sina ;
	m_elements[0][2] = 0.0 ;

	m_elements[1][0] = sina ;
	m_elements[1][1] = cosa ;
	m_elements[1][2] = 0.0 ;

	m_elements[2][0] = 0.0 ;
	m_elements[2][1] = 0.0 ;
	m_elements[2][2] = 1.0 ;
}


/**	Sets the elements of this matrix so that it represents a 2D rotation
 *	about 'centre' by the given anti-clockwise angle in radians.
 */
void Matrix33::set_rotation (double a, const Vector2& centre)
{
	double cosa = cos (a) ;
	double sina = sin (a) ;

	m_elements[0][0] = cosa ;
	m_elements[0][1] = -sina ;
	m_elements[0][2] = centre.x()*(1.0 - cosa) + centre.y()*sina ;

	m_elements[1][0] = sina ;
	m_elements[1][1] = cosa ;
	m_elements[1][2] = centre.y()*(1.0 - cosa) - centre.x()*sina ;

	m_elements[2][0] = 0.0 ;
	m_elements[2][1] = 0.0 ;
	m_elements[2][2] = 1.0 ;
}


/**	Sets the elements of this matrix so that it represents a 2D scale
 *	centred on the origin by the given scale factor.
 */
void Matrix33::set_scale (double s)
{
	m_elements[0][0] = s ;
	m_elements[0][1] = 0.0 ;
	m_elements[0][2] = 0.0 ;

	m_elements[1][0] = 0.0 ;
	m_elements[1][1] = s ;
	m_elements[1][2] = 0.0 ;

	m_elements[2][0] = 0.0 ;
	m_elements[2][1] = 0.0 ;
	m_elements[2][2] = 1.0 ;
}


/**	Sets the elements of this matrix so that it represents a 2D scale
 *	centred on the origin by the factors in X and Y given in the ZVector2.
 */
void Matrix33::set_scale (const Vector2& s)
{
	m_elements[0][0] = s.x() ;
	m_elements[0][1] = 0.0 ;
	m_elements[0][2] = 0.0 ;

	m_elements[1][0] = 0.0 ;
	m_elements[1][1] = s.y() ;
	m_elements[1][2] = 0.0 ;

	m_elements[2][0] = 0.0 ;
	m_elements[2][1] = 0.0 ;
	m_elements[2][2] = 1.0 ;
}


/**	Transforms the given Vector2 by this matrix.
 */
void Matrix33::transform (Vector2* v) const
{
	double x = v->x() ;
	double y = v->y() ;
	v->x() = x*m_elements[0][0] + y*m_elements[0][1] + m_elements[0][2] ;
	v->y() = x*m_elements[1][0] + y*m_elements[1][1] + m_elements[1][2] ;
}


/**	Returns the product of this matrix with the one given.
 */
Matrix33 Matrix33::operator* (const Matrix33& m) const
{
	Matrix33 rtn ;

	for (int r = 0 ; r < 3 ; r++)
	{
		for (int c = 0 ; c < 3 ; c++)
		{
			double v = 0.0 ;
			for (int i = 0 ; i < 3 ; i++)
			{
				v += m_elements[r][i] * m.m_elements[i][c] ;
			}
			rtn.m_elements[r][c] = v ;
		}
	}

	return rtn ;
}


/*########################################################################
	End
########################################################################*/
